import pytest
from aiomoqt.messages import *
from aiomoqt.protocol import *
from aiomoqt.client import *
from aiomoqt.client import *
from aiomoqt.types import *


def test_moqt_stub():
    assert True
